package com.quran.labs.quransearch.data;

import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.os.Environment;


public class DatabaseHandler {
	private SQLiteDatabase database = null;
	
	private static String TL_TABLE = "transliteration";
	private static String TL_COL_VERSE = "versenum";
	private static String TL_COL_SURA = "suranum";
	private static String TL_COL_AYAH = "ayahnum";
	private static String TL_COL_TEXT = "ayahtext";
	
	private static String CACHE_L1_TBL = "cache";
	private static String CACHE_L2_TBL = "cache_l2";
	private static String CACHE_L3_TBL = "cache_l3";
	private static String CACHE_COL_WORD = "word";
	private static String CACHE_COL_SECOND_WORD = "second_word";
	private static String CACHE_COL_THIRD_WORD = "third_word";
	private static String CACHE_COL_MATCHES = "matches";
	
	public DatabaseHandler() throws SQLException {
		String base = DatabaseHandler.getDatabaseDirectory();
		if (base == null) return;
		String path = base + "/transliteration.db";
		database = SQLiteDatabase.openDatabase(path, null,
				SQLiteDatabase.NO_LOCALIZED_COLLATORS);
	}
	
	public boolean validDatabase(){
		return (database == null)? false : database.isOpen();
	}
	
	public Cursor getVerse(int sura, int ayah){
		if (!validDatabase()) return null;
		return database.query(TL_TABLE, new String[]{ TL_COL_VERSE,
				TL_COL_SURA, TL_COL_AYAH, TL_COL_TEXT }, TL_COL_SURA +
				"=" + sura + " and " + TL_COL_AYAH + "=" + ayah, null,
				null, null, null);
	}
	
	public Cursor getVerse(int verse){
		if (!validDatabase()) return null;
		return database.query(TL_TABLE, new String[]{ TL_COL_VERSE,
				TL_COL_SURA, TL_COL_AYAH, TL_COL_TEXT },
				TL_COL_VERSE + "=" + verse, null, null, null, null);
	}
	
	public Cursor getVerses(String verseIds){
		if (!validDatabase()) return null;
		return database.query(TL_TABLE, new String[]{ TL_COL_VERSE,
				TL_COL_SURA, TL_COL_AYAH, TL_COL_TEXT },
				TL_COL_VERSE + " in(" + verseIds + ")",
				null, null, null, null);
	}
	
	public Cursor search(String word){
		if (!validDatabase()) return null;
		return database.query(CACHE_L1_TBL, new String[]{ CACHE_COL_WORD,
				CACHE_COL_MATCHES }, CACHE_COL_WORD + " like '" + word +
				"%'", null, null, null, null);
	}
	
	public Cursor search(String word1, String word2){
		if (!validDatabase()) return null;
		return database.query(CACHE_L2_TBL, new String[]{ CACHE_COL_WORD,
				CACHE_COL_SECOND_WORD, CACHE_COL_MATCHES }, CACHE_COL_WORD +
				"='" + word1 + "' and " + CACHE_COL_SECOND_WORD + " like '" +
				word2 + "%'", null, null, null, null);
	}
	
	public Cursor search(String word1, String word2, String word3){
		if (!validDatabase()) return null;
		return database.query(CACHE_L3_TBL, new String[]{ CACHE_COL_WORD,
				CACHE_COL_SECOND_WORD, CACHE_COL_THIRD_WORD,
				CACHE_COL_MATCHES }, CACHE_COL_WORD + "='" + word1 +
				"' and " + CACHE_COL_SECOND_WORD + "='" +
				word2 + "' and " + CACHE_COL_THIRD_WORD + " like '" + word3 +
				"%'", null, null, null, null);
	}
	
	public void closeDatabase(){ if (database != null) database.close(); }
	
	public static String getDatabaseDirectory(){
		String state = Environment.getExternalStorageState();
		if (state.equals(Environment.MEDIA_MOUNTED))
			return Environment.getExternalStorageDirectory() +
			"/quran_android/databases";
		else return null;
	}
}
