package com.quran.labs.quransearch.data;

import java.util.ArrayList;

import android.database.Cursor;

public class QuranData {
	public static int NUM_RESULTS = 10;
	public static String[] SURA_NAMES = {
		"Al-Fatiha", "Al-Baqara", "Aal-E-Imran", "An-Nisa", "Al-Maeda",
		"Al-Anaam", "Al-Araf", "Al-Anfal", "At-Tawba", "Yunus", "Hud",
		"Yusuf", "Ar-Rad", "Ibrahim", "Al-Hijr", "An-Nahl", "Al-Isra",
		"Al-Kahf", "Maryam", "Ta-Ha", "Al-Anbiya", "Al-Hajj", "Al-Mumenoon",
		"An-Noor", "Al-Furqan", "Ash-Shu'ara", "An-Naml", "Al-Qasas",
		"Al-Ankaboot", "Ar-Room", "Luqman", "As-Sajda", "Al-Ahzab", "Saba",
		"Fatir", "Ya-Seen", "As-Saaffat", "Sad", "Az-Zumar", "Ghafir",
		"Fussilat", "Ash-Shura", "Az-Zukhruf", "Ad-Dukhan", "Al-Jathiya",
		"Al-Ahqaf", "Muhammad", "Al-Fath", "Al-Hujraat", "Qaf",
		"Adh-Dhariyat", "At-Tur", "An-Najm", "Al-Qamar", "Ar-Rahman",
		"Al-Waqia", "Al-Hadid", "Al-Mujadila", "Al-Hashr", "Al-Mumtahina",
		"As-Saff", "Al-Jumua", "Al-Munafiqoon", "At-Taghabun", "At-Talaq",
		"At-Tahrim", "Al-Mulk", "Al-Qalam", "Al-Haaqqa", "Al-Maarij", "Nooh",
		"Al-Jinn", "Al-Muzzammil", "Al-Muddaththir", "Al-Qiyama", "Al-Insan",
		"Al-Mursalat", "An-Naba", "An-Naziat", "Abasa", "At-Takwir",
		"Al-Infitar", "Al-Mutaffifin", "Al-Inshiqaq", "Al-Burooj", "At-Tariq",
		"Al-Ala", "Al-Ghashiya", "Al-Fajr", "Al-Balad", "Ash-Shams",
		"Al-Lail", "Ad-Dhuha", "Al-Inshirah", "At-Tin", "Al-Alaq", "Al-Qadr",
		"Al-Bayyina", "Az-Zalzala", "Al-Adiyat", "Al-Qaria", "At-Takathur",
		"Al-Asr", "Al-Humaza", "Al-Fil", "Quraish", "Al-Maun", "Al-Kauther",
		"Al-Kafiroon", "An-Nasr", "Al-Masadd", "Al-Ikhlas", "Al-Falaq",
		"An-Nas"
	};
	
	public static String getSuraName(int suraNumber){
		if ((suraNumber < 1) || (suraNumber > 114)) return ""; 
		return SURA_NAMES[suraNumber-1];
	}
	
	public ArrayList<Verse> search(String query){
		Cursor res = null;
		DatabaseHandler handler = new DatabaseHandler();
		
		String[] parts = query.split(" ");
		int portions = parts.length;
		if (portions == 1)
			res = handler.search(parts[0]);
		else if (portions == 2)
			res = handler.search(parts[0], parts[1]);
		else {
			res = handler.search(parts[portions-3],
					parts[portions-2], parts[portions-1]);
			portions = 3;
		}
		
		if ((res == null) || (!res.moveToFirst())){
			if (res != null) res.close();
			handler.closeDatabase();
			return new ArrayList<Verse>();
		}

		int unique = 0;
		ArrayList<String[]> matches = new ArrayList<String[]>();
		do {
			unique++;
			String matchStr = res.getString(portions);
			String[] verses = matchStr.split(",");
			matches.add(verses);
		}
		while (res.moveToNext() && (unique < NUM_RESULTS));
		res.close();

		ArrayList<Verse> results = this.chooseResults(matches, handler);
		handler.closeDatabase();
		
		for (Verse v : results)
			v.highlight(query);
		return results;
	}
	
	private ArrayList<Verse> chooseResults(ArrayList<String[]> matches,
			DatabaseHandler handler){
		ArrayList<Verse> results = new ArrayList<Verse>();
		int idx = 0;
		int added = 0;
		int total = 0;
		String matchStr = "";
		do {
			added = 0;
			for (String[] verses : matches){
				if (verses.length > idx){
					if ((added + total) > 0) matchStr += ",";
					matchStr += verses[idx];
					added++;
				}
			}
			
			total += added;
			idx++;
		}
		while ((added > 0) && (total < NUM_RESULTS));
		
		if (matchStr.length() > 0){
			Cursor res = handler.getVerses(matchStr);
			if ((res != null) && (res.moveToFirst())){
				do {
					int verseId = res.getInt(0);
					int suraId = res.getInt(1);
					int ayahId = res.getInt(2);
					String text = res.getString(3);
					results.add(new Verse(verseId, suraId, ayahId, text));
				}
				while (res.moveToNext());
			}
			res.close();
		}
		
		return results;
	}
	
}
